classdef dates<handle

% Copyright (C) 2014-2020 Dynare Team
%
% This code is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% Dynare dates submodule is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.

properties
    freq = []; % Frequency (integer scalar)
    time = []; % Array (one row for every date. first column is the year, second is the period)
end

methods
    function o = dates(varargin)
        if ~nargin
            % Returns empty dates object.
            o.freq = NaN(0);
            o.time = NaN(0,2);
            return
        end
        if all(cellfun(@isdates, varargin))
            % Concatenates dates objects in a dates object.
            o = horzcat(varargin{:});
            return
        end
        if all(cellfun(@isstringdate,varargin))
            % Concatenates dates in a dates object.
            tmp = cellfun(@string2date,varargin);
            if all([tmp.freq]-tmp(1).freq==0)
                o.freq = tmp(1).freq;
            else
                error('dates:ArgCheck', 'All dates passed as inputs must have the same frequency!')
            end
            
            if any(cellfun(@(c)(size(c,2)==3), {tmp.time}, 'UniformOutput', true))
                o.time = transpose(reshape([tmp.time], 3, length(tmp)));
            else
                o.time = transpose(reshape([tmp.time], 2, length(tmp)));
            end
            return
        end
        if isequal(nargin,1) && isfreq(varargin{1})
            % Instantiate an empty dates object (only set frequency)
            o.time = NaN(0,2);
            if ischar(varargin{1})
                o.freq = string2freq(varargin{1});
            else
                o.freq = varargin{1};
            end
            return
        end
        if isequal(nargin,3) && isfreq(varargin{1})
            o.time = NaN(0,2);
            if ischar(varargin{1})
                o.freq = string2freq(varargin{1});
            else
                o.freq = varargin{1};
            end
            if (isnumeric(varargin{2}) && isvector(varargin{2}) && all(isint(varargin{2})))
                if isnumeric(varargin{3}) && isvector(varargin{3}) && all(isint(varargin{3}))
                    if all(varargin{3}>=1) && all(varargin{3}<=o.freq)
                        o.time = [varargin{2}(:), varargin{3}(:)];
                    else
                        error('dates:ArgCheck','Third input must contain integers between 1 and %i.', o.freq)
                    end
                else
                    error('dates:ArgCheck','Third input must be a vector of integers.')
                end
            else
                error('dates:ArgCheck','Second input must be a vector of integers.')
            end
            return
        end
        if isequal(nargin,2) && isfreq(varargin{1})
            o.time = NaN(0,2);
            if ischar(varargin{1})
                o.freq = string2freq(varargin{1});
            else
                o.freq = varargin{1};
            end
            if isequal(o.freq, 1)
                if (isnumeric(varargin{2}) && isvector(varargin{2}) && all(isint(varargin{2})))
                    o.time = [varargin{2}, ones(length(varargin{2}),1)];
                    return
                else
                    error('dates:ArgCheck','Second input must be a vector of integers.')
                end
            else
                if isequal(size(varargin{2},2), 2)
                    if all(isint(varargin{2}(:,1))) && all(isint(varargin{2}(:,2)))
                        if all(varargin{2}(:,2)>=1) && all(varargin{2}(:,2)<=o.freq)
                            o.time = [varargin{2}(:,1), varargin{2}(:,2)];
                        else
                            error('dates:ArgCheck','Second column of the last input must contain integers between 1 and %i.',o.freq)
                        end
                    else
                        error('dates:ArgCheck','Second input argument must be an array of integers.')
                    end
                else
                    error('dates:ArgCheck','Wrong calling sequence!')
                end
            end
            return
        end
        if isequal(nargin,4) && isfreq(varargin{1}) && isequal(varargin{1},365)
            o.time = NaN(0,3);
            if ischar(varargin{1})
                o.freq = string2freq(varargin{1});
            else
                o.freq = varargin{1};
            end
            if (isnumeric(varargin{2}) && isvector(varargin{2}) && all(isint(varargin{2})))
                if isnumeric(varargin{3}) && isvector(varargin{3}) && all(isint(varargin{3}))
                    if isnumeric(varargin{4}) && isvector(varargin{4}) && all(isint(varargin{4}))
                        if all(varargin{3}>=1) && all(varargin{3}<=12) && all(varargin{4}>=1) && all(varargin{3}<=31)
                            o.time = [varargin{2}(:), varargin{3}(:), varargin{4}(:)];
                        else
                            error('dates:ArgCheck','Third input must contain integers between 1 and %i.', o.freq)
                        end
                    else
                        error('dates:ArgCheck','Fourth input must be a vector of integers.')
                    end
                else
                    error('dates:ArgCheck','Third input must be a vector of integers.')
                end
            else
                error('dates:ArgCheck','Second input must be a vector of integers.')
            end
            return
        end
        error('dates:ArgCheck','The input cannot be interpreted as a date. You should first read the manual!')
    end % dates constructor.
        % Other methods
    n = ndat(o);
end % methods
end % classdef
