function date = string2date(a)

% Copyright (C) 2013-2020 Dynare Team
%
% This file is part of Dynare.
%
% Dynare is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% Dynare is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.

date = struct('freq', NaN, 'time', NaN(1,2));

if ~ischar(a) || ~isdate(a)
    error('dates::string2date: Input must be a string that can be interpreted as a date!');
end

if isyearly(a)
    year = 1:(regexp(a,'[AaYy]')-1);
    date.freq = 1;
    date.time = write_time_field_y(a, year);
    return
end

if isbiannual(a)
    year = 1:(regexp(a,'[Hh]')-1);
    date.freq = 2;
    date.time = write_time_field(a, year);
    return
end

if isquarterly(a)
    year = 1:(regexp(a,'[Qq]')-1);
    date.freq = 4;
    date.time = write_time_field(a, year);
    return
end

if ismonthly(a)
    year = 1:(regexp(a,'[Mm]')-1);
    date.freq = 12;
    date.time = write_time_field(a, year);
    return
end

if isweekly(a)
    year = 1:(regexp(a,'[Ww]')-1);
    date.freq = 52;
    date.time = write_time_field(a, year);
    return
end

if isdaily(a)
    year = 1:(regexp(a,'[-]')-1);
    month = 6:7;
    day = 9:10;
    date.freq = 365;
    date.time = write_time_field_d(a, year, month, day);
    return
end

function b = write_time_field(c, d)
b(1) = str2double(c(d));
b(2) = str2double(c(d(end)+2:end));

function b = write_time_field_y(c, d)
b(1) = str2double(c(d));
b(2) = 1;

function b = write_time_field_d(c, y, m, d)
b(1) = str2double(c(y));
b(2) = str2double(c(m));
b(3) = str2double(c(d));
